//package org.apache.lucene.demo;

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import edu.sdsc.grid.io.GeneralFile;
import edu.sdsc.grid.io.srb.SRBAccount;
import edu.sdsc.grid.io.srb.SRBFileSystem;
import edu.sdsc.grid.io.srb.SRBFile;
import edu.sdsc.grid.io.srb.SRBFileInputStream;
import edu.sdsc.grid.io.srb.SRBFileOutputStream;
import edu.sdsc.grid.io.srb.SRBException;
import edu.sdsc.grid.io.local.LocalFile;

import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermEnum;
import org.apache.lucene.store.SRBDirectory;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

import java.io.File;
import java.util.Date;
import java.util.Arrays;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.StringTokenizer;
import java.net.Socket;
import java.net.ServerSocket;

import java.lang.Process;
import java.lang.Runtime;
import java.io.BufferedReader;
import java.io.InputStreamReader;

/** Indexer for HTML files stored on SRB. */
public class Indexer {
  public Indexer() {}

  private static boolean deleting = false;	  // true during deletion pass
  private static IndexReader reader;		  // existing index
  private static IndexWriter writer;		  // new index being built
  private static TermEnum uidIter;		  // document id iterator
  static SRBAccount srbAccount = null;
  static SRBFileSystem srbFileSystem = null;
  static SRBFile srbFile = null;
  static SRBFile srbDir = null;
  static SRBFileInputStream srbFileIn = null;  
  static GeneralFile[] SRBfiles;
  static LocalFile corpusDirFile;
  static File INDEX_DIR;
  static String SRBdirName = " ";
  static boolean merge = false;
  static SRBFile indexonmachine = null;
  static boolean onSimba= false;
  static boolean onDizzy= false;
  static Directory tempdir = null;



  /** Indexer for HTML files.*/
  public static void start(String[] argv) throws Exception {
      String index = "_index_";
      boolean create = false;
      File root = null;
      String SRBCorpusDir = " ";
      LocalFile Env = new LocalFile(".MdasEnv");
      LocalFile Auth = new LocalFile(".MdasAuth"); 
      SRBHandler handler = new SRBHandler(Env,Auth);
      srbAccount = handler.getSRBAccount();
      String  queryserver="none";
      String host = "null";
      String usage = "IndexHTML [-create] [-index <index>] <root_directory>";

      System.out.println("ARGV: "+ argv.length);
      if (argv.length == 0) {
        System.err.println("Usage: " + usage);
        return;
      }

      for (int i = 0; i < argv.length; i++) {
        if (argv[i].equals("-index")) {		  // parse -index option
          index = argv[++i];
        } else if (argv[i].equals("-create")) {	  // parse -create option
          create = true;
        }else if(argv[i].equals("-qserver")){
           queryserver=argv[++i];
        }else if(argv[i].equals("-host")){
           host=argv[++i];
        }else if (i != argv.length-1) {
          System.err.println("Usage: " + usage);
          return;
        } else
          SRBCorpusDir = argv[i];
          //root = new File(srbFileSystem, argv[i]);
      }

      /* Check if index already exists on this machine, if not then create a new one else merge indices*/
      String localvault = new StringTokenizer(java.net.InetAddress.getLocalHost().getHostName(),".").nextToken();
      System.out.println("Local vault: "+ localvault);
      if( ((java.net.InetAddress.getLocalHost().getHostName()).indexOf(".cluster.aim")) >=0){
        onSimba=true;
        //StringTokenizer st = new StringTokenizer(SRBCorpusDir,"/");
        //SRBCorpusDir = st.nextToken() + "/"+st.nextToken(); //user the data on the cluster
      }
      else if(((java.net.InetAddress.getLocalHost().getHostName()).indexOf("sarabi") >=0)){
        onSimba= true;
        System.out.println("Simba");
      }

      if(onSimba){
        srbAccount.setHost(java.net.InetAddress.getLocalHost().getHostName());
        srbAccount.setDefaultStorageResource(localvault);
      }
      else{
        srbAccount.setHost("sarabi.cs.uct.ac.za");
        srbAccount.setDefaultStorageResource("sarabi");
      }
      srbAccount.setHost(java.net.InetAddress.getLocalHost().getHostName());
      srbAccount.setDefaultStorageResource(localvault);
      srbFileSystem = new SRBFileSystem(srbAccount);

       StringTokenizer st = new StringTokenizer(SRBCorpusDir,"/");
       st.nextToken();
      String tempname= "index_"+localvault+"_"+queryserver+"_"+st.nextToken();
      INDEX_DIR = new File(tempname);
      SRBdirName = tempname+".zip"; //compressed index
      indexonmachine = new SRBFile(srbFileSystem, SRBdirName);


//       if(islast.equals("yes")){
//         if(onSimba){ //let indexer node know that indexing cycle has ended
//              int nodeport = 9097;
//             Socket client;         
//             client = new Socket(java.net.InetAddress.getLocalHost().getHostName(),nodeport);
//             PrintWriter socketout = new PrintWriter(client.getOutputStream(), true);
//             socketout.println("3");
//             socketout.flush();
//             socketout.close();
//         }
//         else{ //notify manager about new index so it can move it to sarabi cluster
//             Socket client;
//             int indexmanagerPort = 9096; int nodeport = 9097;
//             System.out.println("Notifying sarabi about new index");
//             client = new Socket ("sarabi.cs.uct.ac.za",indexmanagerPort);  //index manager is on sarabi
//             PrintWriter socketout = new PrintWriter(client.getOutputStream(), true);
//             socketout.println(SRBdirName);
// 	    socketout.flush();
//             client.close();
//             socketout.close();
// 
//             // inform this indexer node that it is end of cycle
//         }
// 
//       }
//      else{ 
        merge = false;

        srbDir = new SRBFile(srbFileSystem, SRBCorpusDir);
        corpusDirFile = new LocalFile(SRBCorpusDir);
        try {
            System.out.println("Copying files from SRB...\r");
            long copyStart      = new Date().getTime();
            srbDir.copyTo(corpusDirFile, true, true);
            long  copyEnd  = new Date().getTime();
            float copySecs = (float) (copyEnd - copyStart) / 1000; 
            File zipped = corpusDirFile.getFile();
            //we have a a tar file untar it and proceed
            Unzip unzip = new Unzip();
            String nameofdir = new StringTokenizer(zipped.getName(),".").nextToken();
            nameofdir = unzip.unzip(SRBCorpusDir);
            root = new File(nameofdir);

            System.out.println("Finished copying files from SRB in "+ copySecs + " secs");
        } catch (NullPointerException e) {
            throw new FileNotFoundException("Cannot open: " + 
                srbDir.getAbsolutePath());
        }

        Date start = new Date();
        tempdir = FSDirectory.getDirectory(INDEX_DIR);

        if (!create) {				  // delete stale docs
            deleting = true;
            indexDocs(root,tempdir , create);
        }
        writer = new IndexWriter(tempdir, new StandardAnalyzer(), create);
        writer.setMaxFieldLength(1000000);
        indexDocs(root,tempdir, create);		  // add new docs

        System.out.println("Optimizing index...");
        writer.optimize();
        writer.close();

        Date end = new Date();
        System.out.print(end.getTime() - start.getTime());
        System.out.println(" total milliseconds");

        //compress the index here
        Zip compress = new Zip();
        compress.zipDir(tempname+".zip",tempname);
        File filetowrite= new File(tempname+".zip");
	 if(!merge){
	   System.out.println("copying index to SRB");
	   writeToSRB(filetowrite,2);
	 }
        filetowrite.delete();
// 	 else if(merge){
// 	        		System.out.println("Merging index on SRB");
// 	            // first get the index off SRB
// 	            LocalFile locali = new LocalFile(indexonmachine.getName());
// 	            indexonmachine.copyTo(locali);
// 	            Directory theindexdir = FSDirectory.getDirectory(locali.getFile());
// 	            IndexWriter writer2 = new IndexWriter(theindexdir, new StandardAnalyzer(), false);
// 	            Directory [] thedirs = new Directory[]{tempdir};
//                     try{
// 	                writer2.addIndexes(thedirs);
//                         writer2.close();
//                     }catch(Exception ex){ex.printStackTrace();
//                         writer2.close();
//                     }
// 	            finally{ 
//                         if(writer2 !=null)
//                             writer2.close();
//                     }
// 	            deleteAllFiles(indexonmachine); //delete old copy of index from SRB
// 	            locali.copyTo(new SRBFile(srbFileSystem,locali.getName()));
//                     System.out.println("Done merging!");
// 	  }


       //ask for more chunks
       int allocatorport = 9097;
       String request = "chunkcompleted:"+ host;
       Socket allocator = new Socket("sarabi.cs.uct.ac.za", allocatorport);
       PrintWriter socketout = new PrintWriter(allocator.getOutputStream(), true);
       socketout.println(request);
       socketout.flush();
       socketout.close();
       allocator.close();

      /* int indexnodeport = 9097;
        Socket inode = new Socket("null",indexnodeport );
        PrintWriter socketout = new PrintWriter(inode.getOutputStream(), true);
        socketout.println("2");
        inode.close();
        socketout.close();*/
     //}
  }

  /* Walk directory hierarchy in uid order, while keeping uid iterator from
  /* existing index in sync.  Mismatches indicate one of: (a) old documents to
  /* be deleted; (b) unchanged documents, to be left alone; or (c) new
  /* documents, to be indexed.
   */

  private static void indexDocs(File file, Directory indexdir, boolean create)
       throws Exception {
    if (!create) {				  // incrementally update

      reader = IndexReader.open(indexdir);		  // open existing index
      uidIter = reader.terms(new Term("uid", "")); // init uid iterator

      indexDocs(file);

      if (deleting) {				  // delete rest of stale docs
        while (uidIter.term() != null && uidIter.term().field() == "uid") {
          System.out.println("deleting " +
              HTMLDocument.uid2url(uidIter.term().text()));
          reader.deleteDocuments(uidIter.term());
          uidIter.next();
        }
        deleting = false;
      }

      uidIter.close();				  // close uid iterator
      reader.close();				  // close existing index

    } else					  // don't have exisiting
      indexDocs(file);
  }

    private static void writeToSRB(File filetowrite, int numReps) {

    try {

      LocalFile Env = new LocalFile(".MdasEnv");
      LocalFile Auth = new LocalFile(".MdasAuth"); 
      SRBHandler md = new SRBHandler(Env, Auth);
      //SRBHandler md = new SRBHandler();
      String localvault = new StringTokenizer(java.net.InetAddress.getLocalHost().getHostName(),".").nextToken();
      srbAccount = md.getSRBAccount();

      if(onSimba){
        srbAccount.setHost(java.net.InetAddress.getLocalHost().getHostName());
        srbAccount.setDefaultStorageResource(localvault);
      }
      else if(onDizzy){
        srbAccount.setHost("dizzy.lcc.uct.ac.za");
        srbAccount.setDefaultStorageResource("dizzy");
      }
     else{
        srbAccount.setHost("sarabi.cs.uct.ac.za");
        srbAccount.setDefaultStorageResource("lacie");
     }
     SRBFileSystem srbFS = new SRBFileSystem(srbAccount);

      /*if (fromSRB) {
          SRBdirName = INDEX_DIR.getName() + 
              "_xd";/* + Integer.toString(maxToIndex) + 
              "_r" + Integer.toString(numReps);                
      /*} else {
          SRBdirName = INDEX_DIR.getName() + 
              "_d" + Integer.toString(maxToIndex) + 
              "_r" + Integer.toString(numReps);                
      }*/

      LocalFile localIndex = new LocalFile(filetowrite);
      System.out.println("\nlocal name: "+ INDEX_DIR.getName());
      localIndex.copyTo(new SRBFile(srbFS, SRBdirName));
      System.out.println("\nremote name: "+ SRBdirName);
      /*String tempindex = SRBdirName+"Temp";
      localIndex.copyTo(new SRBFile( srbFS, tempindex));*/

      /*String[][] definableMetaDataValues = new String[2][2];
      definableMetaDataValues[0][0] = "index";
      definableMetaDataValues[0][1] = "true";           
      definableMetaDataValues[1][0] = "resource";
      definableMetaDataValues[1][1] = localvault;  
      System.out.println("\nLOCALVAULT: "+ localvault);           
     
      md.addDirectoryMetaData(SRBdirName, definableMetaDataValues);*/

    } catch (IOException e) {
        e.printStackTrace();
    }

  }
  private static void indexDocs(File file) throws Exception {
    String name = file.getName();
    int dotIndex = name.indexOf(".");
    String ext ="";
    if ((dotIndex > 0) && (dotIndex < name.length())){
        ext = name.substring(dotIndex + 1, name.length());
    }
    //System.out.println("name:	"+ name+ "extention: "+ext);
    
    if (file.isDirectory()) {			  // if a directory
      String[] files = file.list();		  // list its files
      Arrays.sort(files);			  // sort the files
      for (int i = 0; i < files.length; i++)	  // recursively index them
        indexDocs(new File(file, files[i]));

    } else if ((file.getPath().indexOf(".htm") >0) ||
	   (file.getPath().indexOf(".php") >0) ||
      ext.equalsIgnoreCase("htm") || // index .htm, 
      ext.equalsIgnoreCase("html") || //.html files
      ext.equalsIgnoreCase("php")) { // index .php files

        try{					  // creating a new index
            Document doc = HTMLDocument.Document(file);
           // System.out.println("adding HTML " + doc.get("path"));
            writer.addDocument(doc);		  // add docs unconditionally
        }catch(Exception e){
            System.out.println("Could not add doc");
             //throw new FileNotFoundException("Cannot add document");
        }
    }
    else if(ext.equalsIgnoreCase("pdf")){  //index pdf files
	 	  try{
        Document doc = PDFDocument.Document(file);
        //System.out.println("adding PDF " + doc.get("path"));
        writer.addDocument(doc);
		  }catch(Exception e){System.out.println("An error for PDF");e.printStackTrace();}
    }
    else if(ext.equalsIgnoreCase("doc")){
        try{
		  Document doc = WordDocument.Document(file);
       // System.out.println("adding Word " + doc.get("path"));
        writer.addDocument(doc);
		  }catch(Exception e){System.out.println("An error for Word");e.printStackTrace();}
    }
    else if(ext.equalsIgnoreCase("txt")){
        try{
		  Document doc = PlainTextDocument.Document(file);
        //System.out.println("adding TXT " + doc.get("path"));
        writer.addDocument(doc);
		  }catch(Exception e){System.out.println("An error for TXT");e.printStackTrace();}
    }
    /*else if(ext.equalsIgnoreCase("rtf")){
	 	  try{
        Document doc = RTFDocument.Document(file);
        System.out.println("adding RTF " + doc.get("path"));
        writer.addDocument(doc);
		  }catch(Exception e){System.out.println("An error for PDF");e.printStackTrace();}
    }*/
   }

    /** Delete all files and directory */
    public static boolean deleteAllFiles(SRBFile dir) { 
        if(!dir.exists()) 
            return true; 

        boolean res = true; 
        if(dir.isDirectory()) {
            GeneralFile[] files = dir.listFiles();
            for(int i = 0; i < files.length; i++) 
                res &= deleteAllFiles((SRBFile)files[i]);
            res = dir.delete();//Delete dir itself
        } else 
            res = dir.delete(); 
        return res;
    }
}
