package indexjob;

import edu.sdsc.grid.io.GeneralFile;
import edu.sdsc.grid.io.srb.SRBAccount;
import edu.sdsc.grid.io.srb.SRBFileSystem;
import edu.sdsc.grid.io.srb.SRBFile;
import edu.sdsc.grid.io.srb.SRBFileInputStream;
import edu.sdsc.grid.io.srb.SRBFileOutputStream;
import edu.sdsc.grid.io.srb.SRBException;
import edu.sdsc.grid.io.local.LocalFile;
import edu.sdsc.grid.io.srb.SRBRandomAccessFile;

import java.util.Stack;
import java.util.Iterator;
import java.util.HashMap;
import java.lang.Process;
import java.lang.Runtime;
import java.util.Vector;
import java.io.File;
import java.io.BufferedReader;
import java.io.PrintWriter;
import java.io.FileReader;
import java.net.Socket;
import java.net.ServerSocket;
import java.io.InputStreamReader;
import java.io.FileWriter;
import java.io.BufferedWriter;
import java.util.StringTokenizer;
import java.net.InetAddress;

import java.util.Date;

/**
* Keeps track of partitions to be indexed
* Sends indexing jobs to Indexernodes
* @author ndapandula nakashole
**/
public class DataAllocator{

    static String [] files;
    static  String collection;
    static SRBHandler handler;
    static long dirsize;
    static int numtimes;
    static int numcolls;
    static int numqueryservers;
    public static void main(String [] args)throws Exception{

        if(args.length ==4){
            collection= args[0];
            numcolls = Integer.parseInt(args[1]);       //number of collections to index
            numqueryservers =  Integer.parseInt(args[2]); //number of query servers to store index
            numtimes = Integer.parseInt(args[3]);       // get average over numtimes
        }
        else{  
        System.out.println("Usage: DataAllocator <collection> <numcolls> <numqueryservers> <numtimes>")  ;
        return;         
        }

        //keep track of nodes currently executing jobs
        //key is the hostname, value is <jobid: runningtime> 
        HashMap activenodes = new HashMap();

        Stack listofchunks = new Stack();
        Vector knownservers = new Vector();
        int nodeport = 9017;
        int allocatorport = 9097;
        int SRBserverport = 8097;

        handler = new SRBHandler();
        SRBFile corpus = new SRBFile(handler.getSRBFileSystem(), (collection));
        files = corpus.list();
        for(int i = 0; i <files.length; i++){
            StringTokenizer ss = new StringTokenizer(files[i],".");
            listofchunks.push(ss.nextToken());
        }

        //fill list of chunks otherwise
        if(files.length==1 &&(numcolls>1)){
            while(listofchunks.size() < numcolls){
                StringTokenizer ss = new StringTokenizer(files[0],".");
                listofchunks.push(ss.nextToken());
            }
        }

        /*System.out.println("Ingoring..."+ toignore);
        for(int i =0; i < toignore; i++){
            listofchunks.pop();
        }*/
        System.out.println("Chunks to index: "+ listofchunks.size());

        BufferedReader serverFile = new BufferedReader(new FileReader(new File("/home/nnakasho/apps/GriSAD/indexjob/indexservers")));
        int totservers = Integer.parseInt(serverFile.readLine());
        String [] indexservers = new String[totservers];
        for(int i =0; i < totservers; i++){
            indexservers[i] = serverFile.readLine();
        }
        serverFile.close();

        String line = null;
        ServerSocket server = new ServerSocket(allocatorport, 100);
        BufferedReader in;PrintWriter out ; String chunk, host, cmd;
        StringTokenizer st; 
        boolean done = false,newcycle = true;
        int number = 1;
        int total = listofchunks.size();
		 
        // timing variables
        int indexserver =0,SRBserver=0;
        long start=0, end, start_stopwatch, stop_stopwatch;
        float secs=0, elapsed = 0, indexingtime=0, totalindexingtime=0;
        int staticindexed=0, dynamicindexed =0,indexedchunks =0;


        for(int j =0; j < numtimes; j++){
            //wait for requests
            while(!done){
                System.out.println("Wait for requests ...");
                //check if all activeservers are active
                Iterator it = activenodes.keySet().iterator(); 
                String [] nodes = new String[activenodes.size()]; //create here to prevent ConcurrentAccess Exception
                for(int node=0; node < nodes.length;node++){
                    nodes[node] = (String)it.next();
                }

               for(int node=0; node < nodes.length;node++){
                    String host_as_key = nodes[node];
                    InetAddress addr = InetAddress.getByName(host_as_key);
                    if(!addr.isReachable(120000)){
                        System.out.println(host_as_key+" is gone..");
                        //Put back it's chunck in the queue
                        Vector value = 
                        (Vector)activenodes.get(host_as_key);
                        listofchunks.push((String)value.elementAt(0));
                        activenodes.remove(host_as_key);
                        indexedchunks--;
                        number--;
                    }
                    else{
                        Vector value = 
                        (Vector)activenodes.get(host_as_key);
                        double running_time = (Double)value.elementAt(1)+ elapsed;
                        System.out.print("Running time: "+ running_time);
                        if(running_time >= 900){// 15 mins .. change later
                            //Put back it's chunk in the queue
                            listofchunks.push((String)value.elementAt(0));
                            activenodes.remove(host_as_key);
                            indexedchunks--;
                            number--;
                            System.out.println(host_as_key+" is probably gone..");
                        }else{
                        System.out.println(host_as_key+" is still here..");

                        Vector valuenew = new Vector();
                        valuenew.addElement((String)value.elementAt(0));
                        valuenew.addElement(new Double(running_time));
                        activenodes.put(host_as_key, valuenew);
                        }
                    }

                }
                start_stopwatch= new Date().getTime();
                Socket client = server.accept();

                indexserver++;
                if(indexserver==1)
                    start = new Date().getTime();

                in = new BufferedReader(new InputStreamReader(client.getInputStream()));
                line = in.readLine(); 
                st = new StringTokenizer(line, ":"); //line is of format CMD:HOST
                cmd = st.nextToken();
                host= st.nextToken();

                if(!knownservers.contains(host))
                    knownservers.addElement(host);
                if(cmd.equals("chunkcompleted")){ //node has completed indexing and is requesting for another chunk
                        indexingtime = Float.parseFloat(st.nextToken());
                        totalindexingtime+=indexingtime;
                        System.out.println("Chunks completed \tindex server: "+indexservers[SRBserver] + "\tTime: "+ indexingtime);
                        //notify one of the servers
                        Socket soc = new Socket(indexservers[SRBserver], 8097);
                        PrintWriter pw = new PrintWriter(soc.getOutputStream());
                        String s = host+":"+st.nextToken();
                        pw.println(s);
                        pw.flush();
                        soc.close();
                        pw.close();
                        SRBserver++;
                        if(SRBserver >=totservers)
                            SRBserver =0;
                        //the chunk has completed indexing.
                        //this is added here to ensure that any last indices are waited on by sarabi
                        indexedchunks++;
                        if(host.endsWith(".cluster.aim"))
                            staticindexed++;
                        else
                            dynamicindexed++;
                        activenodes.remove(host);
                        if(indexedchunks==numcolls){
                                end =new Date().getTime();
                                secs+= (float) (end - start) / 1000;
                                printReport((float) (end - start) / 1000,"Time to index round!");
                                done= true;

                        }
 
                }else {

                    if(listofchunks.empty()){ //end of indexing cycle
                        System.out.println("\tEnd of cycle!");
                        //createSubmitFile("none","none",host, "yes");
                    }
                    else{

                        System.out.println("Allocating "+ number +"/"+total +"\t"+host);
                        chunk = (String) listofchunks.pop();
                        System.out.println("Chunk: "+ chunk);
                        Socket soc = new Socket(host, 9017);
                        PrintWriter pw = new PrintWriter(soc.getOutputStream());
                        String s = collection+":"+chunk+":"+host+":"+indexservers[SRBserver];
                        pw.println(s);
                        pw.flush();
                        soc.close();
                        pw.close();


                        Vector value = new Vector();
                        value.addElement(chunk);
                        value.addElement(new Double(0));
                        activenodes.put(host,value);
                        number++;
                    }
                    System.out.println("ACTive nodes:"+ activenodes.size());
                    stop_stopwatch= new Date().getTime();
                    elapsed = (float)(stop_stopwatch - start_stopwatch) / 1000;
                }


            }//end of while


            //notify clients of new cycle
            for(int k=0;k<knownservers.size();k++){
                System.out.println("Notifying: "+(String)knownservers.elementAt(k)+ " of new cycle");
                Socket soc = new Socket((String)knownservers.elementAt(k), nodeport);
                PrintWriter pw = new PrintWriter(soc.getOutputStream());
                pw.println("3");
                pw.flush();

            }
            knownservers.clear();
            number =1;
            indexedchunks=0;
            indexserver=0;
            done = false;
        }
        printReport(secs/(long)numtimes," Average Time to index");
        System.out.println("Total indexed by static machines: "+ staticindexed +"/"+numcolls);
        System.out.println("Total indexed by dynamic machines: "+ dynamicindexed +"/"+numcolls);
        System.out.println("Time spent on actual indexing: "+totalindexingtime);
    }
	 
	 
    public static void createSubmitFile(String coll, String subcoll, String host, String queryserver)throws Exception{
        String fname="/home/nnakasho/apps/GriSAD/"+subcoll+".submit";
        PrintWriter out2 = new PrintWriter(new BufferedWriter(new FileWriter(fname, false)));
        String description = "Universe = java\nExecutable = /home/nnakasho/apps/GriSAD/indexjob/indexjob.class\nLog = indexjob.log.$(Cluster)\nOutput = indexjob.out.$(Cluster).$(Process)\nError ="+ "indexjob.error.$(Cluster)\nRequirements = machine =!= \"sarabi.cs.uct.ac.za\" \nShould_transfer_files = YES\nWhenToTransferOutput = ON_EXIT\ntransfer_input_files = /home/nnakasho/.srb/.MdasEnv,/home/nnakasho/.srb/.MdasAuth \njar_files = "+"/home/nnakasho/apps/lib/jargon_v1.4.20.jar,/home/nnakasho/apps/lib/lucene-core-2.1.0.jar,"+"/home/nnakasho/apps/GriSAD/indexjob/requiredfiles/requiredfiles.jar,/home/nnakasho/apps/lib/jargon_v1.4.20.jar,/home/nnakasho/apps/lib/fontbox.jar,/home/nnakasho/apps/lib/PDFBox.jar,/home/nnakasho/apps/lib/tm-extractors.jar,/home/nnakasho/SRB/SRB3_4_2/matrix/lib/activation.jar,/home/nnakasho/apps/lib/tar.jar,/home/nnakasho/apps/lib/bcprov-jdk15-138.jar,/home/nnakasho/apps/lib/bcmail-jdk15-138.jar\nnotification = never\n";

        try{ //cater for machines that send ips
           int isip = Integer.parseInt(new StringTokenizer(host, ".").nextToken());
           description+="Arguments = indexjob "+ coll + " " + subcoll + " " + queryserver+" "+ host+ "\nRequirements = STRINGIP == \""+host+"\"\nQueue\n";
        }
        catch(Exception e){ //if it fails to parse it is a hostname
            description+="Arguments = indexjob "+ coll + " " + subcoll + " " + queryserver+ " "+host+ "\nRequirements = machine == \""+host+"\"\nQueue\n";
        }
        out2.println(description);
        out2.close();

        String cmd = "condor_submit "+subcoll+".submit";
        Process p = Runtime.getRuntime().exec(cmd);

    }

    /**
     *get the server that should be allocated the index
     *resulting from indexing the chunk
     **/
    public static String getQueryServer(String name, int numservers){
        int server = (name.hashCode())%numservers;
        return Integer.toString(server);
    }


    //prints the message and time
    public static void printReport(float time, String message){
            System.out.println(message+" "+ time);
    }

    //get the size of the indexed directory
    public static long getSize(){
        long size = 0;
        String name;
        System.out.println("length: "+files.length);
        for(int i =0; i < files.length;i++){
            name = collection+"/"+files[i];
            dirsize = 0;
            long sz = getSize(new SRBFile(handler.getSRBFileSystem(),name));
            System.out.println("Size of "+name+" "+sz);
            size+=sz;
        }
        return size;
    }
    
    // gets the size of a sub directory
    public static long getSize(SRBFile dir){
        if(!dir.exists()) 
            return 0; 
        if(dir.isDirectory()) {
            //System.out.println("Directory: "+dir.getName());
            GeneralFile[] files2 = dir.listFiles();
            for(int i = 0; i < files2.length; i++)
                getSize((SRBFile)files2[i]);

        } else {
            dirsize+=dir.length();
        }
        return dirsize;
    }

    /** Delete all files and the directory */
    public static boolean deleteAllFiles(SRBFile dir) { 
        if(!dir.exists()) 
            return true; 

        boolean res = true; 
        if(dir.isDirectory()) {
            GeneralFile[] files = dir.listFiles();
            for(int i = 0; i < files.length; i++) 
                res &= deleteAllFiles((SRBFile)files[i]);
            res = dir.delete();//Delete dir itself
        } else 
            res = dir.delete(); 
        return res;
    }

}

