import java.io.File;
import java.io.IOException;
import java.io.FileInputStream;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.DateTools;

import org.pdfbox.pdfparser.PDFParser;
import org.pdfbox.pdmodel.PDDocument;

import org.pdfbox.util.PDFTextStripper;
import org.pdfbox.cos.COSDocument;
import org.pdfbox.pdmodel.PDDocumentInformation;

public class PDFDocument{
    

    static char dirSep = System.getProperty("file.separator").charAt(0);

    public static String uid(File f) {
        // Append path and date into a string in such a way that lexicographic
        // sorting gives the same results as a walk of the file hierarchy.  Thus
        // null (\u0000) is used both to separate directory components and to
        // separate the path from the date.
        return f.getPath().replace(dirSep, '\u0000') +
        "\u0000" +
        DateTools.timeToString(f.lastModified(), DateTools.Resolution.SECOND);
    }

    public static String uid2url(String uid) {
        String url = uid.replace('\u0000', '/');	  // replace nulls with slashes
        return url.substring(0, url.lastIndexOf('/')); // remove date from end
    }

    public static Document Document(File f)throws IOException, InterruptedException  {
        // make a new, empty document
        Document doc = new Document();
    
        // Add the url as a field named "path".  Use a field that is 
        // indexed (i.e. searchable), but don't tokenize the field into words.
        doc.add(new Field("path", f.getPath().replace(dirSep, '/'), Field.Store.YES,
            Field.Index.UN_TOKENIZED));

        doc.add(new Field("title", f.getName(), Field.Store.YES,Field.Index.UN_TOKENIZED));
    
        // Add the last modified date of the file a field named "modified".  
        // Use a field that is indexed (i.e. searchable), but don't tokenize
        // the field into words.
        doc.add(new Field("modified",
            DateTools.timeToString(f.lastModified(), DateTools.Resolution.MINUTE),
            Field.Store.YES, Field.Index.UN_TOKENIZED));
    
        // Add the uid as a field, so that index can be incrementally maintained.
        // This field is not stored with document, it is indexed, but it is not
        // tokenized prior to indexing.
        doc.add(new Field("uid", uid(f), Field.Store.NO, Field.Index.UN_TOKENIZED));

        FileInputStream fis = new FileInputStream(f);
        PDFParser parser = new PDFParser(fis);
        COSDocument cosDoc = null;
        try{
            parser.parse();
            cosDoc = parser.getDocument();  //load inpustream into memory
        }
        catch(IOException e){
           cosDoc.close();
           System.out.println("Error in PDFParser: Parsing document");
           //return null;
        }

        //Extract PDF document textual content
        String docText = null;
        try{
            PDFTextStripper stripper = new PDFTextStripper();
            docText = stripper.getText(new PDDocument(cosDoc));
        }
        catch(IOException e){
            cosDoc.close();
            System.out.println("Error in PDFParser: Getting text");
            //return null;
        }

        if(docText != null){
            doc.add(new Field("contents", docText, Field.Store.NO, Field.Index.TOKENIZED));
            cosDoc.close();
           // doc.add(Field.UnStored("content", docText));
        }
        else{
           cosDoc.close();
        }

        return doc;
    }

    private PDFDocument() {}
}