import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.DateTools;

import java.io.File;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.FileInputStream;
public class PlainTextDocument{

    static char dirSep = System.getProperty("file.separator").charAt(0);

    public static String uid(File f) {
        // Append path and date into a string in such a way that lexicographic
        // sorting gives the same results as a walk of the file hierarchy.  Thus
        // null (\u0000) is used both to separate directory components and to
        // separate the path from the date.
        return f.getPath().replace(dirSep, '\u0000') +
        "\u0000" +
        DateTools.timeToString(f.lastModified(), DateTools.Resolution.SECOND);
    }

    public static String uid2url(String uid) {
        String url = uid.replace('\u0000', '/');	  // replace nulls with slashes
        return url.substring(0, url.lastIndexOf('/')); // remove date from end
    }

    public static Document Document(File f) {
        // make a new, empty document
        Document doc = new Document();
    
        // Add the url as a field named "path".  Use a field that is 
        // indexed (i.e. searchable), but don't tokenize the field into words.
        doc.add(new Field("path", f.getPath().replace(dirSep, '/'), Field.Store.YES,
            Field.Index.UN_TOKENIZED));

        doc.add(new Field("title", f.getName(), Field.Store.YES,Field.Index.UN_TOKENIZED));
    
        // Add the last modified date of the file a field named "modified".  
        // Use a field that is indexed (i.e. searchable), but don't tokenize
        // the field into words.
        doc.add(new Field("modified",
            DateTools.timeToString(f.lastModified(), DateTools.Resolution.MINUTE),
            Field.Store.YES, Field.Index.UN_TOKENIZED));
    
        // Add the uid as a field, so that index can be incrementally maintained.
        // This field is not stored with document, it is indexed, but it is not
        // tokenized prior to indexing.
        doc.add(new Field("uid", uid(f), Field.Store.NO, Field.Index.UN_TOKENIZED));
        
        String docText = "";
        
        try {
            FileInputStream fis = new FileInputStream(f);
            BufferedReader br =
                new BufferedReader(new InputStreamReader(fis));
            String line = null;
            while ((line = br.readLine()) != null) {
                docText += line;
            }
            br.close();
        }
        catch(IOException e) {
            System.out.println("Error in PlainTextDocument: Reading document");
        }
        if (!docText.equals("")) {
            doc.add(new Field("contents", docText, Field.Store.NO, Field.Index.TOKENIZED));
            return doc;
        }
        return null;
    }
} 
